Class: CExtractorTypes::CFunctionDefinition
- Defined in:
- lib/ceedling/c_extractor/c_extractor_types.rb
Overview
Data class representing an extracted C function definition
Instance Attribute Summary collapse
-
#body ⇒ Object
Returns the value of attribute body.
-
#code_block ⇒ Object
Returns the value of attribute code_block.
-
#decorators ⇒ Object
Returns the value of attribute decorators.
-
#line_count ⇒ Object
Returns the value of attribute line_count.
-
#line_num ⇒ Object
Returns the value of attribute line_num.
-
#name ⇒ Object
Returns the value of attribute name.
-
#signature ⇒ Object
Returns the value of attribute signature.
-
#signature_stripped ⇒ Object
Returns the value of attribute signature_stripped.
-
#source_filepath ⇒ Object
Returns the value of attribute source_filepath.
Instance Method Summary collapse
-
#initialize(name: nil, signature: nil, body: nil, code_block: nil, line_count: 0, source_filepath: nil, line_num: nil, decorators: [], signature_stripped: nil) ⇒ CFunctionDefinition
constructor
Constructor to set unassigned fields to nil for convenience.
Constructor Details
#initialize(name: nil, signature: nil, body: nil, code_block: nil, line_count: 0, source_filepath: nil, line_num: nil, decorators: [], signature_stripped: nil) ⇒ CFunctionDefinition
Constructor to set unassigned fields to nil for convenience
90 91 92 93 94 95 96 97 98 99 100 101 102 |
# File 'lib/ceedling/c_extractor/c_extractor_types.rb', line 90 def initialize( name: nil, signature: nil, body: nil, code_block: nil, line_count: 0, source_filepath: nil, line_num: nil, decorators: [], signature_stripped: nil ) super end |
Instance Attribute Details
#body ⇒ Object
Returns the value of attribute body
77 78 79 |
# File 'lib/ceedling/c_extractor/c_extractor_types.rb', line 77 def body @body end |
#code_block ⇒ Object
Returns the value of attribute code_block
77 78 79 |
# File 'lib/ceedling/c_extractor/c_extractor_types.rb', line 77 def code_block @code_block end |
#decorators ⇒ Object
Returns the value of attribute decorators
77 78 79 |
# File 'lib/ceedling/c_extractor/c_extractor_types.rb', line 77 def decorators @decorators end |
#line_count ⇒ Object
Returns the value of attribute line_count
77 78 79 |
# File 'lib/ceedling/c_extractor/c_extractor_types.rb', line 77 def line_count @line_count end |
#line_num ⇒ Object
Returns the value of attribute line_num
77 78 79 |
# File 'lib/ceedling/c_extractor/c_extractor_types.rb', line 77 def line_num @line_num end |
#name ⇒ Object
Returns the value of attribute name
77 78 79 |
# File 'lib/ceedling/c_extractor/c_extractor_types.rb', line 77 def name @name end |
#signature ⇒ Object
Returns the value of attribute signature
77 78 79 |
# File 'lib/ceedling/c_extractor/c_extractor_types.rb', line 77 def signature @signature end |
#signature_stripped ⇒ Object
Returns the value of attribute signature_stripped
77 78 79 |
# File 'lib/ceedling/c_extractor/c_extractor_types.rb', line 77 def signature_stripped @signature_stripped end |
#source_filepath ⇒ Object
Returns the value of attribute source_filepath
77 78 79 |
# File 'lib/ceedling/c_extractor/c_extractor_types.rb', line 77 def source_filepath @source_filepath end |