Class: Partials::FunctionDeclaration
- Defined in:
- lib/ceedling/partials/partials.rb
Overview
Data class representing a C function signature
Instance Attribute Summary collapse
-
#line_num ⇒ Object
Returns the value of attribute line_num.
-
#name ⇒ Object
Returns the value of attribute name.
-
#return_type ⇒ Object
Returns the value of attribute return_type.
-
#signature ⇒ Object
Returns the value of attribute signature.
-
#source_filepath ⇒ Object
Returns the value of attribute source_filepath.
Instance Method Summary collapse
-
#initialize(name: nil, signature: nil, source_filepath: nil, line_num: nil) ⇒ FunctionDeclaration
constructor
Constructor to set unassigned fields to nil for convenience.
Constructor Details
#initialize(name: nil, signature: nil, source_filepath: nil, line_num: nil) ⇒ FunctionDeclaration
Constructor to set unassigned fields to nil for convenience
44 45 46 |
# File 'lib/ceedling/partials/partials.rb', line 44 def initialize(name: nil, signature: nil, source_filepath: nil, line_num: nil) super end |
Instance Attribute Details
#line_num ⇒ Object
Returns the value of attribute line_num
35 36 37 |
# File 'lib/ceedling/partials/partials.rb', line 35 def line_num @line_num end |
#name ⇒ Object
Returns the value of attribute name
35 36 37 |
# File 'lib/ceedling/partials/partials.rb', line 35 def name @name end |
#return_type ⇒ Object
Returns the value of attribute return_type
35 36 37 |
# File 'lib/ceedling/partials/partials.rb', line 35 def return_type @return_type end |
#signature ⇒ Object
Returns the value of attribute signature
35 36 37 |
# File 'lib/ceedling/partials/partials.rb', line 35 def signature @signature end |
#source_filepath ⇒ Object
Returns the value of attribute source_filepath
35 36 37 |
# File 'lib/ceedling/partials/partials.rb', line 35 def source_filepath @source_filepath end |