Class: TestInvokerTypes::Testable

Inherits:
Struct
  • Object
show all
Defined in:
lib/ceedling/test_invoker/test_invoker_types.rb

Overview

Named record replacing the raw hash per test file. Fields are populated across multiple stages; nil fields are valid until their stage sets them.

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(**kwargs) ⇒ Testable

Returns a new instance of Testable.



43
44
45
46
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 43

def initialize(**kwargs)
  kwargs[:partials] ||= TestablePartials.new(configs: {}, tests: [], mocks: [])
  super(**kwargs)
end

Instance Attribute Details

#assembler_flagsObject

Returns the value of attribute assembler_flags

Returns:

  • (Object)

    the current value of assembler_flags



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def assembler_flags
  @assembler_flags
end

#compile_definesObject

Returns the value of attribute compile_defines

Returns:

  • (Object)

    the current value of compile_defines



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def compile_defines
  @compile_defines
end

#compile_flagsObject

Returns the value of attribute compile_flags

Returns:

  • (Object)

    the current value of compile_flags



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def compile_flags
  @compile_flags
end

#coreObject

Returns the value of attribute core

Returns:

  • (Object)

    the current value of core



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def core
  @core
end

#executableObject

Returns the value of attribute executable

Returns:

  • (Object)

    the current value of executable



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def executable
  @executable
end

#filepathObject

Returns the value of attribute filepath

Returns:

  • (Object)

    the current value of filepath



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def filepath
  @filepath
end

#frameworksObject

Returns the value of attribute frameworks

Returns:

  • (Object)

    the current value of frameworks



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def frameworks
  @frameworks
end

Returns the value of attribute link_flags

Returns:

  • (Object)

    the current value of link_flags



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def link_flags
  @link_flags
end

#mocksObject

Returns the value of attribute mocks

Returns:

  • (Object)

    the current value of mocks



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def mocks
  @mocks
end

#nameObject

Returns the value of attribute name

Returns:

  • (Object)

    the current value of name



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def name
  @name
end

Returns the value of attribute no_link_objects

Returns:

  • (Object)

    the current value of no_link_objects



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def no_link_objects
  @no_link_objects
end

#objectsObject

Returns the value of attribute objects

Returns:

  • (Object)

    the current value of objects



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def objects
  @objects
end

#partialsObject

Returns the value of attribute partials

Returns:

  • (Object)

    the current value of partials



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def partials
  @partials
end

#pathsObject

Returns the value of attribute paths

Returns:

  • (Object)

    the current value of paths



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def paths
  @paths
end

#preprocessObject

Returns the value of attribute preprocess

Returns:

  • (Object)

    the current value of preprocess



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def preprocess
  @preprocess
end

#preprocess_definesObject

Returns the value of attribute preprocess_defines

Returns:

  • (Object)

    the current value of preprocess_defines



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def preprocess_defines
  @preprocess_defines
end

#preprocess_flagsObject

Returns the value of attribute preprocess_flags

Returns:

  • (Object)

    the current value of preprocess_flags



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def preprocess_flags
  @preprocess_flags
end

#results_failObject

Returns the value of attribute results_fail

Returns:

  • (Object)

    the current value of results_fail



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def results_fail
  @results_fail
end

#results_passObject

Returns the value of attribute results_pass

Returns:

  • (Object)

    the current value of results_pass



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def results_pass
  @results_pass
end

#runnerObject

Returns the value of attribute runner

Returns:

  • (Object)

    the current value of runner



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def runner
  @runner
end

#search_pathsObject

Returns the value of attribute search_paths

Returns:

  • (Object)

    the current value of search_paths



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def search_paths
  @search_paths
end

#sourcesObject

Returns the value of attribute sources

Returns:

  • (Object)

    the current value of sources



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def sources
  @sources
end

#toolObject

Returns the value of attribute tool

Returns:

  • (Object)

    the current value of tool



29
30
31
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29

def tool
  @tool
end