Class: Partials::FunctionDefinition
- Defined in:
- lib/ceedling/partials/partials.rb
Overview
Data class representing a C function with intentionally duplicated fields
Instance Attribute Summary collapse
-
#code_block ⇒ Object
Returns the value of attribute code_block.
-
#line_num ⇒ Object
Returns the value of attribute line_num.
-
#name ⇒ Object
Returns the value of attribute name.
-
#signature ⇒ Object
Returns the value of attribute signature.
-
#source_filepath ⇒ Object
Returns the value of attribute source_filepath.
Instance Method Summary collapse
-
#initialize(name: nil, signature: nil, code_block: nil, source_filepath: nil, line_num: nil) ⇒ FunctionDefinition
constructor
Constructor to set unassigned fields to nil for convenience.
Constructor Details
#initialize(name: nil, signature: nil, code_block: nil, source_filepath: nil, line_num: nil) ⇒ FunctionDefinition
Constructor to set unassigned fields to nil for convenience
59 60 61 |
# File 'lib/ceedling/partials/partials.rb', line 59 def initialize(name: nil, signature: nil, code_block: nil, source_filepath: nil, line_num: nil) super end |
Instance Attribute Details
#code_block ⇒ Object
Returns the value of attribute code_block
50 51 52 |
# File 'lib/ceedling/partials/partials.rb', line 50 def code_block @code_block end |
#line_num ⇒ Object
Returns the value of attribute line_num
50 51 52 |
# File 'lib/ceedling/partials/partials.rb', line 50 def line_num @line_num end |
#name ⇒ Object
Returns the value of attribute name
50 51 52 |
# File 'lib/ceedling/partials/partials.rb', line 50 def name @name end |
#signature ⇒ Object
Returns the value of attribute signature
50 51 52 |
# File 'lib/ceedling/partials/partials.rb', line 50 def signature @signature end |
#source_filepath ⇒ Object
Returns the value of attribute source_filepath
50 51 52 |
# File 'lib/ceedling/partials/partials.rb', line 50 def source_filepath @source_filepath end |