Class: TestInvokerTypes::Testable
- Defined in:
- lib/ceedling/test_invoker/test_invoker_types.rb
Overview
Named record replacing the raw hash per test file. Fields are populated across multiple stages; nil fields are valid until their stage sets them.
Instance Attribute Summary collapse
-
#assembler_flags ⇒ Object
Returns the value of attribute assembler_flags.
-
#compile_defines ⇒ Object
Returns the value of attribute compile_defines.
-
#compile_flags ⇒ Object
Returns the value of attribute compile_flags.
-
#core ⇒ Object
Returns the value of attribute core.
-
#executable ⇒ Object
Returns the value of attribute executable.
-
#filepath ⇒ Object
Returns the value of attribute filepath.
-
#frameworks ⇒ Object
Returns the value of attribute frameworks.
-
#link_flags ⇒ Object
Returns the value of attribute link_flags.
-
#mocks ⇒ Object
Returns the value of attribute mocks.
-
#name ⇒ Object
Returns the value of attribute name.
-
#no_link_objects ⇒ Object
Returns the value of attribute no_link_objects.
-
#objects ⇒ Object
Returns the value of attribute objects.
-
#partials ⇒ Object
Returns the value of attribute partials.
-
#paths ⇒ Object
Returns the value of attribute paths.
-
#preprocess ⇒ Object
Returns the value of attribute preprocess.
-
#preprocess_defines ⇒ Object
Returns the value of attribute preprocess_defines.
-
#preprocess_flags ⇒ Object
Returns the value of attribute preprocess_flags.
-
#results_fail ⇒ Object
Returns the value of attribute results_fail.
-
#results_pass ⇒ Object
Returns the value of attribute results_pass.
-
#runner ⇒ Object
Returns the value of attribute runner.
-
#search_paths ⇒ Object
Returns the value of attribute search_paths.
-
#sources ⇒ Object
Returns the value of attribute sources.
-
#tool ⇒ Object
Returns the value of attribute tool.
Instance Method Summary collapse
-
#initialize(**kwargs) ⇒ Testable
constructor
A new instance of Testable.
Constructor Details
#initialize(**kwargs) ⇒ Testable
Returns a new instance of Testable.
43 44 45 46 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 43 def initialize(**kwargs) kwargs[:partials] ||= TestablePartials.new(configs: {}, tests: [], mocks: []) super(**kwargs) end |
Instance Attribute Details
#assembler_flags ⇒ Object
Returns the value of attribute assembler_flags
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def assembler_flags @assembler_flags end |
#compile_defines ⇒ Object
Returns the value of attribute compile_defines
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def compile_defines @compile_defines end |
#compile_flags ⇒ Object
Returns the value of attribute compile_flags
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def compile_flags @compile_flags end |
#core ⇒ Object
Returns the value of attribute core
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def core @core end |
#executable ⇒ Object
Returns the value of attribute executable
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def executable @executable end |
#filepath ⇒ Object
Returns the value of attribute filepath
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def filepath @filepath end |
#frameworks ⇒ Object
Returns the value of attribute frameworks
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def frameworks @frameworks end |
#link_flags ⇒ Object
Returns the value of attribute link_flags
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def link_flags @link_flags end |
#mocks ⇒ Object
Returns the value of attribute mocks
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def mocks @mocks end |
#name ⇒ Object
Returns the value of attribute name
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def name @name end |
#no_link_objects ⇒ Object
Returns the value of attribute no_link_objects
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def no_link_objects @no_link_objects end |
#objects ⇒ Object
Returns the value of attribute objects
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def objects @objects end |
#partials ⇒ Object
Returns the value of attribute partials
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def partials @partials end |
#paths ⇒ Object
Returns the value of attribute paths
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def paths @paths end |
#preprocess ⇒ Object
Returns the value of attribute preprocess
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def preprocess @preprocess end |
#preprocess_defines ⇒ Object
Returns the value of attribute preprocess_defines
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def preprocess_defines @preprocess_defines end |
#preprocess_flags ⇒ Object
Returns the value of attribute preprocess_flags
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def preprocess_flags @preprocess_flags end |
#results_fail ⇒ Object
Returns the value of attribute results_fail
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def results_fail @results_fail end |
#results_pass ⇒ Object
Returns the value of attribute results_pass
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def results_pass @results_pass end |
#runner ⇒ Object
Returns the value of attribute runner
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def runner @runner end |
#search_paths ⇒ Object
Returns the value of attribute search_paths
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def search_paths @search_paths end |
#sources ⇒ Object
Returns the value of attribute sources
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def sources @sources end |
#tool ⇒ Object
Returns the value of attribute tool
29 30 31 |
# File 'lib/ceedling/test_invoker/test_invoker_types.rb', line 29 def tool @tool end |